﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.Modeling.Validation;
using Microsoft.VisualStudio.Modeling;

namespace JKU.binarytree_language
{
    [ValidationState(ValidationState.Enabled)]
    public partial class Tree
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save)]
        private void ValidateStatesReachable(ValidationContext context)
        {
            //Find start state
            RootNode startState = null;
            foreach (Node state in this.Elements)
            {
                if (state is RootNode)
                {
                    if (startState == null)
                    {
                        startState = state as RootNode;
                        if (state.Source != null)
                        {
                            context.LogError("It is not allowed to have an incomming transition in a start state " + state.Name, "Err 04", state);
                        }
                        if (state.Targets.Count > 2)
                        {
                            context.LogError("The root node has to have maximum two children(s)!" + state.Name, "", state);
                        }
                    }
                    else
                    {
                        context.LogError("Model contains more than one start state " + state.Name, "Err 01", state);
                    }
                }
            }

            if (startState == null)
            {
                context.LogError("Model should contain exactly one start state", "Err 02", this);
            }

            //Test states reachability
            IList<Node> reachableStates = new List<Node>();
            findReachableStates(startState, reachableStates);

            foreach (Node state in this.Elements)
            {
                if (!reachableStates.Contains(state))
                {
                    context.LogError("State with name " + state.Name + " is unreachable", "Err 03", state);
                }
            }

            //Constraints for the InternalNode
            foreach (Node state in this.Elements)
            {
                if (state is InternalNode)
                {
                       if (state.Targets.Count > 2)
                       {
                           context.LogError("An internal node has to have maximum two childrens!" + state.Name, "", state);
                       }
                       if (state.Targets.Count < 1)
                       {
                           context.LogError("An internal node has to have minimum one children!" + state.Name, "", state);
                       }
                       if (state.Source == null)
                       {
                           context.LogError("An internal node has to have exactly one source!" + state.Name, "Err 02", state);
                       }
                }
                if (state is LeafNode)
                {
                    if (state.Targets.Count != 0)
                    {
                        context.LogError("A leaf node must not have children(s)!" + state.Name, "", state);
                    }
                    if (state.Source == null)
                    {
                        context.LogError("A leaf node has to have exactly one source!" + state.Name, "Err 02", state);
                    }
                }
                
            }
        }

        private void findReachableStates(Node state, IList<Node> reachableStates)
        {
            if (!reachableStates.Contains(state))
            {
                reachableStates.Add(state);
                foreach (Node targetState in state.Targets)
                {
                    findReachableStates(targetState, reachableStates);
                }
            }
        }

     
    }
}